--
-- Livepatch database dump.
--
-- This schema upgrade upgrades the db schema to a state
-- of the last alembic-based schema upgrade.


CREATE TYPE patch_state AS ENUM (
    'applied',
    'apply-failed',
    'unapplied',
    'needs-check',
    'nothing-to-apply',
    'unknown',
    'check-failed',
    'applied-with-bug',
    'kernel-upgrade-required'
);

CREATE TABLE auth_token (
    id integer NOT NULL,
    sso_id character varying NOT NULL,
    auth_token character varying NOT NULL,
    channel integer NOT NULL,
    creation_time timestamp without time zone NOT NULL
);

CREATE SEQUENCE auth_token_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE auth_token_id_seq OWNED BY auth_token.id;

CREATE TABLE channel (
    id integer NOT NULL,
    name character varying NOT NULL
);

CREATE SEQUENCE channel_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE channel_id_seq OWNED BY channel.id;

CREATE TABLE kernel (
    version character varying NOT NULL,
    creation_time timestamp without time zone NOT NULL
);

CREATE TABLE livepatch_dist (
    patch_id integer NOT NULL,
    hashsum character varying NOT NULL,
    channel integer NOT NULL,
    kernel_version character varying NOT NULL,
    filename character varying NOT NULL
);

CREATE TABLE machine (
    id integer NOT NULL,
    creation_time timestamp without time zone NOT NULL,
    uuid character varying NOT NULL,
    testing boolean
);

CREATE SEQUENCE machine_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE machine_id_seq OWNED BY machine.id;

CREATE TABLE machine_token (
    machine_id integer NOT NULL,
    auth_token_id integer NOT NULL,
    machine_token character varying NOT NULL
);

CREATE TABLE metrics (
    created timestamp without time zone NOT NULL,
    metrics jsonb NOT NULL
);

CREATE TABLE on_prem (
    id integer NOT NULL,
    auth_token character varying NOT NULL,
    last_sync date NOT NULL,
    bytes_synced integer
);

CREATE SEQUENCE on_prem_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE on_prem_id_seq OWNED BY on_prem.id;

CREATE TABLE patch (
    id integer NOT NULL,
    creation_time timestamp without time zone NOT NULL,
    version character varying NOT NULL
);

CREATE SEQUENCE patch_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE patch_id_seq OWNED BY patch.id;

CREATE TABLE report (
    id integer NOT NULL,
    creation_time timestamp without time zone NOT NULL,
    machine_id integer NOT NULL,
    last_check timestamp without time zone,
    uptime integer NOT NULL,
    boot_time timestamp without time zone NOT NULL,
    client_version character varying NOT NULL,
    architecture character varying NOT NULL,
    cpu_model character varying NOT NULL,
    channel_name character varying,
    sso_id character varying,
    report_date date,
    testing boolean
);

CREATE SEQUENCE report_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_id_seq OWNED BY report.id;

CREATE TABLE report_kernel (
    running boolean NOT NULL,
    report_id integer NOT NULL,
    kernel_version character varying NOT NULL
);

CREATE TABLE report_kernel_patch (
    patch_id integer NOT NULL,
    state patch_state NOT NULL,
    report_id integer NOT NULL,
    kernel_version character varying NOT NULL
);

ALTER TABLE ONLY auth_token ALTER COLUMN id SET DEFAULT nextval('auth_token_id_seq'::regclass);

ALTER TABLE ONLY channel ALTER COLUMN id SET DEFAULT nextval('channel_id_seq'::regclass);

ALTER TABLE ONLY machine ALTER COLUMN id SET DEFAULT nextval('machine_id_seq'::regclass);

ALTER TABLE ONLY on_prem ALTER COLUMN id SET DEFAULT nextval('on_prem_id_seq'::regclass);

ALTER TABLE ONLY patch ALTER COLUMN id SET DEFAULT nextval('patch_id_seq'::regclass);

ALTER TABLE ONLY report ALTER COLUMN id SET DEFAULT nextval('report_id_seq'::regclass);

ALTER TABLE ONLY auth_token
    ADD CONSTRAINT auth_token_auth_token_key UNIQUE (auth_token);

ALTER TABLE ONLY auth_token
    ADD CONSTRAINT auth_token_pkey PRIMARY KEY (id);

ALTER TABLE ONLY auth_token
    ADD CONSTRAINT auth_token_sso_id_channel_key UNIQUE (sso_id, channel);

ALTER TABLE ONLY channel
    ADD CONSTRAINT channel_name_key UNIQUE (name);

ALTER TABLE ONLY channel
    ADD CONSTRAINT channel_pkey PRIMARY KEY (id);

ALTER TABLE ONLY report_kernel_patch
    ADD CONSTRAINT idx_report_kernel_patch_uniq UNIQUE (report_id, patch_id, kernel_version);

ALTER TABLE ONLY report_kernel
    ADD CONSTRAINT idx_report_kernel_uniq UNIQUE (report_id, kernel_version);

ALTER TABLE ONLY report
    ADD CONSTRAINT idx_report_uniq UNIQUE (report_date, machine_id, client_version);

ALTER TABLE ONLY kernel
    ADD CONSTRAINT kernel_pkey PRIMARY KEY (version);

ALTER TABLE ONLY livepatch_dist
    ADD CONSTRAINT livepatch_dist_kernel_version_patch_id_channel_key UNIQUE (kernel_version, patch_id, channel);

ALTER TABLE ONLY machine
    ADD CONSTRAINT machine_pkey PRIMARY KEY (id);

ALTER TABLE ONLY machine_token
    ADD CONSTRAINT machine_token_machine_id_key UNIQUE (machine_id);

ALTER TABLE ONLY machine_token
    ADD CONSTRAINT machine_token_machine_token_key UNIQUE (machine_token);

ALTER TABLE ONLY machine
    ADD CONSTRAINT machine_uuid_key UNIQUE (uuid);

ALTER TABLE ONLY on_prem
    ADD CONSTRAINT on_prem_pkey PRIMARY KEY (id);

ALTER TABLE ONLY patch
    ADD CONSTRAINT patch_pkey PRIMARY KEY (id);

ALTER TABLE ONLY patch
    ADD CONSTRAINT patch_version_key UNIQUE (version);

ALTER TABLE ONLY report
    ADD CONSTRAINT report_pkey PRIMARY KEY (id);

CREATE INDEX idx_auth_token_auth_token ON auth_token USING btree (auth_token);

CREATE INDEX idx_authtoken_channel ON auth_token USING btree (channel);

CREATE INDEX idx_authtoken_sso_id ON auth_token USING btree (sso_id);

CREATE INDEX idx_machinetoken_authtokenid ON machine_token USING btree (auth_token_id);

CREATE INDEX ix_metrics_created ON metrics USING btree (created);

CREATE INDEX report_kernel_patch_report_id_idx ON report_kernel_patch USING btree (report_id);

CREATE INDEX report_kernel_report_id_idx ON report_kernel USING btree (report_id);

ALTER TABLE ONLY auth_token
    ADD CONSTRAINT auth_token_channel_fkey FOREIGN KEY (channel) REFERENCES channel(id) ON DELETE CASCADE;

ALTER TABLE ONLY livepatch_dist
    ADD CONSTRAINT livepatch_dist_channel_fkey FOREIGN KEY (channel) REFERENCES channel(id) ON DELETE CASCADE;

ALTER TABLE ONLY livepatch_dist
    ADD CONSTRAINT livepatch_dist_kernel_version_fkey FOREIGN KEY (kernel_version) REFERENCES kernel(version);

ALTER TABLE ONLY livepatch_dist
    ADD CONSTRAINT livepatch_dist_patch_id_fkey FOREIGN KEY (patch_id) REFERENCES patch(id) ON DELETE CASCADE;

ALTER TABLE ONLY machine_token
    ADD CONSTRAINT machine_token_auth_token_id_fkey FOREIGN KEY (auth_token_id) REFERENCES auth_token(id) ON DELETE CASCADE;

ALTER TABLE ONLY machine_token
    ADD CONSTRAINT machine_token_machine_id_fkey FOREIGN KEY (machine_id) REFERENCES machine(id) ON DELETE CASCADE;

ALTER TABLE ONLY on_prem
    ADD CONSTRAINT on_prem_auth_token_fkey FOREIGN KEY (auth_token) REFERENCES auth_token(auth_token);

ALTER TABLE ONLY report_kernel
    ADD CONSTRAINT report_kernel_kernel_version_fkey FOREIGN KEY (kernel_version) REFERENCES kernel(version);

ALTER TABLE ONLY report_kernel_patch
    ADD CONSTRAINT report_kernel_patch_kernel_version_fkey FOREIGN KEY (kernel_version) REFERENCES kernel(version);

ALTER TABLE ONLY report_kernel_patch
    ADD CONSTRAINT report_kernel_patch_patch_id_fkey FOREIGN KEY (patch_id) REFERENCES patch(id) ON DELETE CASCADE;

ALTER TABLE ONLY report_kernel_patch
    ADD CONSTRAINT report_kernel_patch_report_id_fkey FOREIGN KEY (report_id) REFERENCES report(id) ON DELETE CASCADE;

ALTER TABLE ONLY report_kernel
    ADD CONSTRAINT report_kernel_report_id_fkey FOREIGN KEY (report_id) REFERENCES report(id) ON DELETE CASCADE;

ALTER TABLE ONLY report
    ADD CONSTRAINT report_machine_id_fkey FOREIGN KEY (machine_id) REFERENCES machine(id) ON DELETE CASCADE;

INSERT INTO channel (name) VALUES ('edge'), ('beta'), ('candidate'), ('stable');

-- Create an index on the report table to improve patch health reports.
CREATE INDEX idx_report_lookup ON report (channel_name, report_date);
