-- This schema migration breaks up the livepatch_dist table.
-- The design of this table is counterintuitive as it contains two types
-- of information:
--  1. which patch files are available in which tiers (channel column)
--  2. patch file information (patch_version, hashsum, kernel_version, filename columns)
--
-- This means that for each tier a patch is associated with, the file's information
-- is duplicated. Since this is a relatively small table and the results of its lookups
-- are cached, the intention is to split it up.

CREATE TABLE patch_file (
    patch_version TEXT NOT NULL REFERENCES patch(version) ON DELETE CASCADE,
    kernel_version TEXT NOT NULL REFERENCES kernel(version),
    hashsum TEXT NOT NULL,
    filename TEXT NOT NULL
);
ALTER TABLE patch_file ADD PRIMARY KEY (kernel_version, patch_version);
INSERT INTO patch_file (patch_version, kernel_version, hashsum, filename)
  SELECT patch_version, kernel_version, hashsum, filename FROM livepatch_dist
  ON CONFLICT (patch_version, kernel_version) DO NOTHING;

-- Create table for associating patch files with tiers.
-- The reason why the association is (patch file) - (tier) and not
-- (patch version) - (tier) is because it is anticipated that improvements
-- to livepatch server will be needed that permit promotion of specific patch
-- files to tiers (instead of promoting all files associated with a patch).
CREATE TABLE patch_file_tier (
       tier_id integer NOT NULL,
       patch_version TEXT NOT NULL,
       kernel_version TEXT NOT NULL,
       FOREIGN KEY (tier_id) REFERENCES channel(id) ON DELETE CASCADE,
       FOREIGN KEY (kernel_version, patch_version)
         REFERENCES patch_file (kernel_version, patch_version)
         ON DELETE CASCADE
);
ALTER TABLE patch_file_tier ADD PRIMARY KEY (tier_id, kernel_version, patch_version);
INSERT INTO patch_file_tier (tier_id, patch_version, kernel_version)
  SELECT channel, patch_version, kernel_version FROM livepatch_dist
  ON CONFLICT (tier_id, kernel_version, patch_version) DO NOTHING;
CREATE INDEX idx_patch_file_tier_tier ON patch_file_tier(tier_id);
